library(readxl)
library(boot)
library(tidyr)
library(nnet)

pdf("Figure5.pdf")
par(mfcol=c(1,2))

#The dataset is obtained from "Dataset 1" in the supplementary material of [Wiinholt et al. 2020]
#https://doi.org/10.1038/s41598-020-65214-7
data <- read_excel("Wiinhold_data__41598_2020_65214_MOESM2_ESM.xlsx", sheet = "data")
data = subset(data,quality=="high")
data = data[,c("mouse","R1a_ctr","R2_ctr","R3a_ctr","R4_ctr")]

m = 4
n = dim(data)[1]

data$m = apply(data[,c("R1a_ctr","R2_ctr","R3a_ctr","R4_ctr")],1,mean)
data$s = apply(data[,c("R1a_ctr","R2_ctr","R3a_ctr","R4_ctr")],1,sd)
bias = apply(data[,c("R1a_ctr","R2_ctr","R3a_ctr","R4_ctr")]-data$m,2,mean)
whichmax =apply(abs(data[,c("R1a_ctr","R2_ctr","R3a_ctr","R4_ctr")]-data$m),1,which.is.max)
s = mean(data$s)

L = sqrt(qchisq(0.95,m-1)) * 1 / sqrt(m-1) * s

#m denoted mm to avoid naming conflict with parameter of boot
funL <- function(data,indices,mm){
  d <- data[indices,]
  d$s = apply(d[,c("R1a_ctr","R2_ctr","R3a_ctr","R4_ctr")],1,sd)
  s = mean(d$s)
  return(sqrt(qchisq(0.95,mm-1)) * 1 / sqrt(mm-1) * s)
}

b = boot(data=data,statistic=funL,R=1000,mm=4)
ci = boot.ci(b, type="bca")
Lll = ci$bca[4]
Lul = ci$bca[5]

plot(data$m,rep(NA,times=n),ylim=c(0,1.2),main=paste0("Wiinholt et al."), pch=20,xlab="mean",ylab="standard deviation",xaxt="n")
for(i in 1:m){
  points(subset(data$m,whichmax==i),subset(data$s,whichmax==i),pch=20,col=i)
}
for(i in 1:m){
  axis(4,at=abs(bias[i]),lab=NA,col.ticks=i)
}
legend("top",pch=20,col=1:m,legend=1:m,ncol=m,bty="n",cex=.7)
axis(1,at=seq(125,160,5))

abline(L,0)
abline(Lll,0,lty=3)
abline(Lul,0,lty=3)




#The data set is obtained from the R package MethComp
#https://cran.r-project.org/web/packages/MethComp/index.html
load("glucose.rda")
data0 = glucose
data0 = subset(data0,time==0)
data0 = data0[,c("type","item","y")]
data0 = data0[!duplicated(data0[,c("item","type")]),]
data = data0 %>% pivot_wider(names_from = type, values_from = y)
data = subset(data,!is.na(plasma)&!is.na(blood)&!is.na(capil)&!is.na(serum))

m = 4  
n = dim(data)[1]

data$m = apply(data[,c("plasma","blood","capil","serum")],1,mean)
data$s = apply(data[,c("plasma","blood","capil","serum")],1,sd)
bias = apply(data[,c("plasma","blood","capil","serum")]-data$m,2,mean)
whichmax =apply(abs(data[,c("plasma","blood","capil","serum")]-data$m),1,which.is.max)
s = mean(data$s)

L = sqrt(qchisq(0.95,m-1)) * 1 / sqrt(m-1) * s

#m denoted mm to avoid naming conflict with parameter of boot
funL <- function(data,indices,mm){
  d <- data[indices,]
  d$s = apply(d[,c("plasma","blood","capil","serum")],1,sd)
  s = mean(d$s)
  return(sqrt(qchisq(0.95,mm-1)) * 1 / sqrt(mm-1) * s)
}

b = boot(data=data,statistic=funL,R=1000,mm=4)
ci = boot.ci(b, type="bca")
Lll = ci$bca[4]
Lul = ci$bca[5]

plot(data$m,rep(NA,times=n),ylim=c(0,1.4),main=paste0("Carstensen et al."), pch=20,xlab="mean",ylab="standard deviation",xaxt="n")
for(i in 1:m){
  points(subset(data$m,whichmax==i),subset(data$s,whichmax==i),pch=20,col=i)
}
for(i in 1:m){
  axis(4,at=abs(bias[i]),lab=NA,col.ticks=i)
}
legend("top",pch=20,col=1:m,legend=1:m,ncol=m,bty="n",cex=.7)
axis(1,at=seq(4.5,7,0.5))

abline(L,0)
abline(Lll,0,lty=3)
abline(Lul,0,lty=3)
dev.off()

