library(nnet)
library(boot)

M = 5
strue = 2

limfun <- function(raters, data, indices){
	m=raters
	dw2 <- dw[indices,] # allows boot to select sample
	means=sqrt(mean(dw2$s^2))
	return(sqrt(qchisq(0.95,m-1))*sqrt(m-1)/(m-1)*means)
}

pdf("Figure1.pdf")
par(mfcol=c(2,3))
for(n in c(10,20,100)){
  set.seed(1234)
  d = data.frame(id = rep(1:n,each=M) )
  d$rep = rep(1:M,times=n)
  d$g = rep(runif(n,0,10),each=M)
  d$e = rnorm(n*M,0,strue)
  d$x = d$g + d$e
  
  dw = reshape(d[,c("id","rep","x")],idvar="id",timevar="rep",direction="wide")[,2:(M+1)]
  
  for(m in 2){
    dw$d = dw[,1]-dw[,2]
    dw$s = apply(dw[,1:m],1,sd)
    dw$m = apply(dw[,1:m],1,mean)
    means=sqrt(mean(dw$s^2))
    lim=sqrt(qchisq(0.95,m-1))*sqrt(m-1)/(m-1)*means
    results <- boot(data=dw, statistic=limfun,R=1000, raters=m)  
    ci=boot.ci(results, type="bca")
    if(n==10) ymax=6
    if(n==20) ymax=6
    if(n==100) ymax=10
    plot(dw$m,dw$s,ylim=c(0,10),xlim=c(-4,12),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standard deviation",xaxt="n",yaxt="n")
    axis(1,at=seq(-4,12,4))
    axis(2,at=seq(0,10,2),las=2)
    abline(lim,0)
    abline(ci$bca[4],0,lty=3)
    abline(ci$bca[5],0,lty=3)
    
    limBA=1.96*means*sqrt(2)*sqrt(1+1/n)
    
    #These quantiles are from [Carkeet 2015, Appendix Table 2] (note that v = n-1)
    if(n==10){
      lower = 1.3915
      upper = 3.7706
    }
    if(n==20){
      lower = 1.5175
      upper = 2.9445
    }
    if(n==100){
      lower = 1.7283
      upper = 2.2896
    }
    
    limBAlower=lower*means*sqrt(2)*sqrt(1+1/n)
    limBAupper=upper*means*sqrt(2)*sqrt(1+1/n)
    
    maxx=max(limBA,dw$d,-dw$d)
    plot(dw$m,dw$d,ylim=c(-14,14),xlim=c(-4,12),main=substitute(atop("Classical BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab=expression(paste("difference = standard deviation"/sqrt(2))),xaxt="n",yaxt="n")
    abline(-limBA,0)
    abline(limBA,0)
    abline(0,0,lty="dashed")
    abline(limBAlower,0,lty=3)
    abline(limBAupper,0,lty=3)
    abline(-limBAlower,0,lty=3)
    abline(-limBAupper,0,lty=3)
    axis(1,at=seq(-4,12,4))
    axis(2,at=seq(-14,14,2),las=2)
  }
}
dev.off()


pdf("Figure2.pdf")
par(mfrow=c(3,4))
for(n in c(10,20,100)){
set.seed(1234)
d = data.frame(id = rep(1:n,each=M) )
d$rep = rep(1:M,times=n)
d$g = rep(runif(n,0,10),each=M)
d$e = rnorm(n*M,0,strue)
d$x = d$g + d$e

dw = reshape(d[,c("id","rep","x")],idvar="id",timevar="rep",direction="wide")[,2:(M+1)]

for(m in 2:5){
dw$d = dw[,1]-dw[,2]
dw$s = apply(dw[,1:m],1,sd)
dw$m = apply(dw[,1:m],1,mean)
bias = apply(dw[,1:m]-dw$m,2,mean)
whichmax =apply(abs(dw[,1:m]-dw$m),1,which.is.max)
means=sqrt(mean(dw$s^2))
lim=sqrt(qchisq(0.95,m-1))*sqrt(m-1)/(m-1)*means

results <- boot(data=dw, statistic=limfun,R=1000, raters=m)  
ci=boot.ci(results, type="bca")

if(n==10) ymax=6
if(n==20) ymax=6
if(n==100) ymax=10
if(m==2){
plot(dw$m,dw$s,xlim=c(-4,12),ylim=c(0,ymax),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standard deviation",xaxt="n")
axis(1,at=seq(-4,12,4))
abline(lim,0)
abline(ci$bca[4],0,lty=3)
abline(ci$bca[5],0,lty=3)
}
if(m>2){
  plot(dw$m,rep(NA,times=n),xlim=c(-4,12),ylim=c(0,ymax),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standard deviation",xaxt="n")
for(i in 1:m){
  points(subset(dw$m,whichmax==i),subset(dw$s,whichmax==i),pch=20)
}
  axis(1,at=seq(-4,12,4))
  abline(lim,0)
  abline(ci$bca[4],0,lty=3)
  abline(ci$bca[5],0,lty=3)
}

}
}
dev.off()




pdf("Figure3.pdf")
par(mfrow=c(3,3))
for(n in 100){
  set.seed(1234)
  d = data.frame(id = rep(1:n,each=M) )
  d$rep = rep(1:M,times=n)
  d$g = rep(runif(n,0,10),each=M)
  d$e = rnorm(n*M,0,strue)
  d$x = d$g + d$e
  
  dw = reshape(d[,c("id","rep","x")],idvar="id",timevar="rep",direction="wide")[,2:(M+1)]
  
  for(m in c(3,4,5)){
    dw$d = dw[,1]-dw[,2]
    dw$s = apply(dw[,1:m],1,sd)
    dw$m = apply(dw[,1:m],1,mean)
    bias = apply(dw[,1:m]-dw$m,2,mean)
    whichmax =apply(abs(dw[,1:m]-dw$m),1,which.is.max)
    means=sqrt(mean(dw$s^2))
    lim=sqrt(qchisq(0.95,m-1))*sqrt(m-1)/(m-1)*means
    results <- boot(data=dw, statistic=limfun,R=1000, raters=m)  
    ci=boot.ci(results, type="bca")
    
    if(m==2){
      plot(dw$m,dw$s,ylim=c(0,max(lim,dw$s,log10(n)*5)),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standard deviation")
      abline(lim,0)
    }
    if(m>2){
      plot(dw$m,rep(NA,times=n),ylim=c(0,max(lim,dw$s,log10(n)*5)+1),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standard deviation")
      for(i in 1:m){
        points(subset(dw$m,whichmax==i),subset(dw$s,whichmax==i),pch=20,col=i)
      }
      abline(lim,0)
      abline(ci$bca[4],0,lty=3)
      abline(ci$bca[5],0,lty=3)
      
    }
    
    abline(lim,0)
    if(m==2) axis(4,at=abs(bias[1]),lab=NA,col.ticks=1)
    if(m>2){
      for(i in 1:m){
        axis(4,at=abs(bias[i]),lab=NA,col.ticks=i)
      }
      legend("top",pch=20,col=1:i,legend=1:i,ncol=i,bty="n",cex=.7)
    }
  }
}

for(n in 100){
  set.seed(1234)
  d = data.frame(id = rep(1:n,each=M) )
  d$rep = rep(1:M,times=n)
  d$g = rep(runif(n,0,10),each=M)
  d$e = rnorm(n*M,0,strue)
  d$x = d$g + d$e
  
  dw = reshape(d[,c("id","rep","x")],idvar="id",timevar="rep",direction="wide")[,2:(M+1)]
  
  #Introduce bias for rater 2
  dw[,2] = dw[,2]+2
  
  for(m in c(3,4,5)){
    dw$d = dw[,1]-dw[,2]
    dw$s = apply(dw[,1:m],1,sd)
    dw$m = apply(dw[,1:m],1,mean)
    bias = apply(dw[,1:m]-dw$m,2,mean)
    whichmax =apply(abs(dw[,1:m]-dw$m),1,which.is.max)
    means=sqrt(mean(dw$s^2))
    lim=sqrt(qchisq(0.95,m-1))*sqrt(m-1)/(m-1)*means
    results <- boot(data=dw, statistic=limfun,R=1000, raters=m)  
    ci=boot.ci(results, type="bca")
    
    if(m==2){
      plot(dw$m,dw$s,ylim=c(0,max(lim,dw$s,log10(n)*5)),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standard deviation")
      abline(lim,0)
      abline(ci$bca[4],0,lty=3)
      abline(ci$bca[5],0,lty=3)
      
    }
    if(m>2){
      plot(dw$m,rep(NA,times=n),ylim=c(0,max(lim,dw$s,log10(n)*5)+1),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standard deviation")
      for(i in 1:m){
        points(subset(dw$m,whichmax==i),subset(dw$s,whichmax==i),pch=20,col=i)
      }
      abline(lim,0)
      abline(ci$bca[4],0,lty=3)
      abline(ci$bca[5],0,lty=3)
      
    }
    
    abline(lim,0)
    abline(ci$bca[4],0,lty=3)
    abline(ci$bca[5],0,lty=3)
    
    if(m==2) axis(4,at=abs(bias[1]),lab=NA,col.ticks=1)
    if(m>2){
      for(i in 1:m){
        axis(4,at=abs(bias[i]),lab=NA,col.ticks=i)
      }
      legend("top",pch=20,col=1:m,legend=1:m,ncol=m,bty="n",cex=.7)
    }
  }
}

for(n in 100){
  set.seed(1234)
  d = data.frame(id = rep(1:n,each=M) )
  d$rep = rep(1:M,times=n)
  d$g = rep(runif(n,0,10),each=M)
  d$e = rnorm(n*M,0,strue)
  d$x = d$g + d$e
  
  dw = reshape(d[,c("id","rep","x")],idvar="id",timevar="rep",direction="wide")[,2:(M+1)]
  
  #Introduce additional uncertainty for rater 2
  dw[,2] = dw[,2]+rnorm(n,0,3*strue)
  
  for(m in c(3,4,5)){
    dw$d = dw[,1]-dw[,2]
    dw$s = apply(dw[,1:m],1,sd)
    dw$m = apply(dw[,1:m],1,mean)
    bias = apply(dw[,1:m]-dw$m,2,mean)
    whichmax =apply(abs(dw[,1:m]-dw$m),1,which.is.max)
    means=sqrt(mean(dw$s^2))
    lim=sqrt(qchisq(0.95,m-1))*sqrt(m-1)/(m-1)*means
    results <- boot(data=dw, statistic=limfun,R=1000, raters=m)  
    ci=boot.ci(results, type="bca")
    
    if(m==2){
      plot(dw$m,dw$s,ylim=c(0,max(lim,dw$s,log10(n)*5)),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standard deviation")
      abline(lim,0)
    }
    if(m>2){
      plot(dw$m,rep(NA,times=n),ylim=c(0,max(lim,dw$s,log10(n)*5)+1),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standard deviation")
      for(i in 1:m){
        points(subset(dw$m,whichmax==i),subset(dw$s,whichmax==i),pch=20,col=i)
      }
      abline(lim,0)
      abline(ci$bca[4],0,lty=3)
      abline(ci$bca[5],0,lty=3)
      
    }
    
    abline(lim,0)
    abline(ci$bca[4],0,lty=3)
    abline(ci$bca[5],0,lty=3)
    
    if(m==2) axis(4,at=abs(bias[1]),lab=NA,col.ticks=1)
    if(m>2){
      for(i in 1:m){
        axis(4,at=abs(bias[i]),lab=NA,col.ticks=i)
      }
      legend("top",pch=20,col=1:m,legend=1:m,ncol=m,bty="n",cex=.7)
    }
  }
}
dev.off()




pdf("Figure4.pdf")
par(mfcol=c(2,3))
for(n in c(10,20,100)){
set.seed(1234)
d = data.frame(id = rep(1:n,each=M) )
d$rep = rep(1:M,times=n)
d$g = rep(runif(n,0,10),each=M)
d$e = d$g/5*rnorm(n*M,0,strue)
d$x = d$g + d$e

dw = reshape(d[,c("id","rep","x")],idvar="id",timevar="rep",direction="wide")[,2:(M+1)]

for(m in 2){
dw$d = dw[,1]-dw[,2]
dw$s = apply(dw[,1:m],1,sd)
dw$m = apply(dw[,1:m],1,mean)
means=sqrt(mean(dw$s^2))
lim=sqrt(qchisq(0.95,m-1))*sqrt(m-1)/(m-1)*means
results <- boot(data=dw, statistic=limfun,R=1000, raters=m)  
ci=boot.ci(results, type="bca")

plot(dw$m,dw$s,xlim=c(0,15),ylim=c(0,12),main=substitute(atop("Extended BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab="standardd deviation",xaxt="n")
axis(1,at=seq(0,15,5))
abline(lim,0)
abline(ci$bca[4],0,lty=3)
abline(ci$bca[5],0,lty=3)
limBA=1.96*means*sqrt(2)*sqrt(1+1/n)

#These quantiles are from [Carkeet 2015, Appendix Table 2] (note that v = n-1)
if(n==10){
  lower = 1.3915
  upper = 3.7706
}
if(n==20){
  lower = 1.5175
  upper = 2.9445
}
if(n==100){
  lower = 1.7283
  upper = 2.2896
}

limBAlower=lower*means*sqrt(2)*sqrt(1+1/n)
limBAupper=upper*means*sqrt(2)*sqrt(1+1/n)

maxx=max(limBA,dw$d,-dw$d)
plot(dw$m,dw$d,xlim=c(0,15),ylim=c(-12,12),main=substitute(atop("Classical BA plot",paste( italic("m"), " = ", mp, " ", italic("n"), " = ", np)),list(mp=m,np=n)), pch=20,xlab="mean",ylab=expression(paste("standard deviation" == sqrt(2)*difference)),xaxt="n")
axis(1,at=seq(0,15,5))
abline(-limBA,0)
abline(limBA,0)
abline(limBAlower,0,lty=3)
abline(limBAupper,0,lty=3)
abline(-limBAlower,0,lty=3)
abline(-limBAupper,0,lty=3)
abline(0,0)
}
}
dev.off()
